require "mechanize"

module Mobistar
  class CookieError < Exception; end
  class AuthenticationError < Exception; end
  class MissingLink < Exception; end
  class InvalidGSMNumber < Exception; end
  class NotLoggedIn < Exception; end
  
  class Number
    def initialize string
      if string =~ /^(\+32|0)4([7-9]\d{7})$/
        @num = $2
      else
        raise InvalidGSMNumber, "#{string} isn't a valid belgian gsm number !"
      end
    end
    
    def international
      "+324#{@num}"
    end
    
    def belgian
      "04#{@num}"
    end
    
    def human group=2
      "04#{@num[0..1]}/#{@num[2..-1].gsub(/(\d{#{group}})/, '\1.')[0...-1]}"
    end
  end
  
  class Client
    LOGIN_URL = "https://res.mobistar.be/imfrontend/dwr/call/plaincall/LoginDwr.checkLogin.dwr?callCount=1&page=%2Ffr%2Fe-services%2Flogin%3FTYPE%3D33554433%26REALMOID%3D06-38929314-8a51-1125-95a9-84e323580cb3%26GUID%3D%26SMAUTHREASON%3D0%26METHOD%3DGET%26SMAGENTNAME%3D%24SM%244T0F5sTwaomxEpeIVaniBrJhN3p12h%252fZ1tlR7msTW8vDiSUgQxpGjnQ5sd3em5iO%26TARGET%3D%24SM%24https%253a%252f%252fmy%252emobistar%252ebe%252fsecured%252fespace-client%252findex%252ehtml&httpSessionId=&scriptSessionId=A542EDD0B6E72E6CE3C3C3D8C3E8F464740&c0-scriptName=LoginDwr&c0-methodName=checkLogin&c0-id=0&c0-e1=string%3A%{USERNAME}&c0-e2=string%3A%{PASSWORD}&c0-e3=string%3Afr&c0-e4=string%3A%2524SM%2524https%25253a%25252f%25252fmy%25252emobistar%25252ebe%25252fsecured%25252fespace-client%25252findex%25252ehtml&c0-e5=string%3Amobistarresidential&c0-param0=Object_Object%3A%7Blogin%3Areference%3Ac0-e1%2C%20password%3Areference%3Ac0-e2%2C%20language%3Areference%3Ac0-e3%2C%20targetUrl%3Areference%3Ac0-e4%2C%20referrer%3Areference%3Ac0-e5%7D&batchId=0"
    
    def initialize username, password
      @loggedin = nil
      @agent = Mechanize.new do |ag|
        ag.follow_meta_refresh = true
      end
      if block_given?
        loginDWR username, password
        yield self
      end
    end
    
    def send_sms number, message
      number_parsed = number.kind_of?(Number) ? number : Number.new(number)
      number = number_parsed.belgian #Ensure valid number
      
      raise NotLoggedIn unless @loggedin
      link = @loggedin.link_with :text => 'Envoyer un e-mail'
      raise MissingLink unless link
      send_email = @agent.click link
      link = send_email.link_with :text => /crire un SMS$/
      raise MissingLink unless link

      #Go to SMS Page and use form
      send_sms = @agent.click link
      confirmation = send_sms.form_with(:name => "formulaire"){|form|
        form['NUMTEL'] = number
        form['listetel'] = ',,'+number
        form['corpsms'] = form['msg'] = message
        form['typesms'] = '2'
        form['produit'] = '1000'
        form['delai'] = 'now'
      }.submit.form_with(:name=>'formulaire').submit
      return true
    end
    
    protected
    def loginDWR username, password
      url = LOGIN_URL.gsub(/%\{USERNAME\}/, username).gsub(/%\{PASSWORD\}/, password)
      loginDWR = @agent.get url
      
      if loginDWR.body =~ /dwr.engine._remoteHandleCallback\('0','0',\{(.+)\}\);$/
        response = {}
        $1.split(/,/).each do |keyval| 
          key, val = keyval.split(/:/, 2)
          val = val[1...-1] if val.start_with?('"') && val.end_with?('"')
          val = nil if val == "null"
          val = false if val == "false"
          val = true if val == "true"
          response[key] = val
        end
      else
        raise AuthenticationError
      end
      
      auth_cookie = Mechanize::Cookie.new 'SMSESSION', response['cookieSMValue']
      auth_cookie.domain = '.mobistar.be'
      auth_cookie.path = '/'
      cook = @agent.cookie_jar.add @agent.history.last.uri, auth_cookie
      raise CookieError, "Cannot add auth cookie to session !!!" unless cook
      @loggedin = @agent.get response['targetURL']
      return true
    end
  end
end
